@@ -17,7 +17,9 @@
  * License along with this library; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
- */
+*/
+
+#include <config.h>
 #include <string.h>
 #include <stdio.h>
 #include "tasklist.h"
@@ -62,6 +64,14 @@ typedef struct _WnckTaskClass   WnckTaskClass;
 
 #define N_SCREEN_CONNECTIONS 4
 
+typedef enum
+{
+  WNCK_TASK_APPLICATION,
+  WNCK_TASK_WINDOW,
+  WNCK_TASK_STARTUP_SEQUENCE
+
+} WnckTaskType;
+
 struct _WnckTask
 {
   GObject parent_instance;
@@ -71,10 +81,15 @@ struct _WnckTask
   GtkWidget *button;
   GtkWidget *image;
   GtkWidget *label;
+
+  WnckTaskType type;
   
   WnckApplication *application;
   WnckWindow *window;
-
+#ifdef HAVE_STARTUP_NOTIFICATION
+  SnStartupSequence *startup_sequence;
+#endif
+  
   gdouble grouping_score;
 
   GList *windows; /* List of the WnckTask for the window,
@@ -89,7 +104,6 @@ struct _WnckTask
   /* ops menu */
   GtkWidget *action_menu;
 
-  guint is_application : 1;  /* TRUE if this task is a grouped application */
   guint really_toggling : 1; /* Set when tasklist really wants
                               * to change the togglebutton state
                               */
@@ -115,6 +129,9 @@ struct _WnckTasklistPrivate
   /* Calculated by update_lists */
   GList *windows;
   GList *applications;
+
+  /* Not handled by update_lists */
+  GList *startup_sequences;
   
   GHashTable *win_hash;
   GHashTable *app_hash;
@@ -137,6 +154,10 @@ struct _WnckTasklistPrivate
 
   gint minimum_width;
   gint minimum_height;
+
+#ifdef HAVE_STARTUP_NOTIFICATION
+  SnMonitorContext *sn_context;
+#endif
 };
 
 
@@ -148,6 +169,10 @@ static WnckTask *wnck_task_new_from_window      (WnckTasklist    *tasklist,
 						 WnckWindow      *window);
 static WnckTask *wnck_task_new_from_application (WnckTasklist    *tasklist,
 						 WnckApplication *application);
+#ifdef HAVE_STARTUP_NOTIFICATION
+static WnckTask *wnck_task_new_from_startup_sequence (WnckTasklist      *tasklist,
+                                                      SnStartupSequence *sequence);
+#endif
 
 static char      *wnck_task_get_text (WnckTask *task);
 static GdkPixbuf *wnck_task_get_icon (WnckTask *task);
@@ -164,6 +189,8 @@ static void     wnck_tasklist_size_request  (GtkWidget        *widget,
                                              GtkRequisition   *requisition);
 static void     wnck_tasklist_size_allocate (GtkWidget        *widget,
                                              GtkAllocation    *allocation);
+static void     wnck_tasklist_realize       (GtkWidget        *widget);
+static void     wnck_tasklist_unrealize     (GtkWidget        *widget);
 static void     wnck_tasklist_forall        (GtkContainer     *container,
                                              gboolean	       include_internals,
                                              GtkCallback       callback,
@@ -202,6 +229,12 @@ static void     wnck_tasklist_connect_screen           (WnckTasklist *tasklist,
                                                         WnckScreen   *screen);
 static void     wnck_tasklist_disconnect_screen        (WnckTasklist *tasklist);
 
+#ifdef HAVE_STARTUP_NOTIFICATION
+static void     wnck_tasklist_sn_event                 (SnMonitorEvent *event,
+                                                        void           *user_data);
+#endif
+
+
 static gpointer task_parent_class;
 static gpointer tasklist_parent_class;
 
@@ -276,11 +309,8 @@ wnck_task_finalize (GObject *object)
       task->button = NULL;
     }
 
-  if (task->is_application)
-    {
-      g_list_free (task->windows);
-      task->windows = NULL;
-    }
+  g_list_free (task->windows);
+  task->windows = NULL;
 
   if (task->state_changed_tag)
     {
@@ -334,6 +364,14 @@ wnck_task_finalize (GObject *object)
       task->application = NULL;
     }
 
+#ifdef HAVE_STARTUP_NOTIFICATION
+  if (task->startup_sequence)
+    {
+      sn_startup_sequence_unref (task->startup_sequence);
+      task->startup_sequence = NULL;
+    }
+#endif
+  
   if (task->button_activate != 0)
     {
       g_source_remove (task->button_activate);
@@ -405,7 +443,9 @@ wnck_tasklist_class_init (WnckTasklistClass *klass)
 
   widget_class->size_request = wnck_tasklist_size_request;
   widget_class->size_allocate = wnck_tasklist_size_allocate;
-
+  widget_class->realize = wnck_tasklist_realize;
+  widget_class->unrealize = wnck_tasklist_unrealize;
+  
   container_class->forall = wnck_tasklist_forall;
   container_class->remove = wnck_tasklist_remove;
 }
@@ -424,6 +464,7 @@ wnck_tasklist_finalize (GObject *object)
    */
   g_assert (tasklist->priv->windows == NULL);
   g_assert (tasklist->priv->applications == NULL);
+  g_assert (tasklist->priv->startup_sequences == NULL);
   /* wnck_tasklist_free_tasks (tasklist); */
   
   g_hash_table_destroy (tasklist->priv->win_hash);
@@ -679,6 +720,7 @@ wnck_tasklist_size_request  (GtkWidget      *widget,
   GArray *array;
   GList *ungrouped_apps;
   int n_windows;
+  int n_startup_sequences;
   int n_rows;
   int n_cols, last_n_cols;
   int n_grouped_buttons;
@@ -721,6 +763,21 @@ wnck_tasklist_size_request  (GtkWidget      *widget,
       l = l->next;
     }
 
+  l = tasklist->priv->startup_sequences;
+  while (l != NULL)
+    {
+      WnckTask *task = WNCK_TASK (l->data);
+      
+      gtk_widget_size_request (task->button, &child_req);
+      
+      max_height = MAX (child_req.height,
+			max_height);
+      max_width = MAX (child_req.width,
+		       max_width);
+      
+      l = l->next;
+    }
+  
   tasklist->priv->max_button_width = max_width;
   tasklist->priv->max_button_height = max_height;
 
@@ -756,6 +813,7 @@ wnck_tasklist_size_request  (GtkWidget      *widget,
   /* Calculate size_hints list */
   
   n_windows = g_list_length (tasklist->priv->windows);
+  n_startup_sequences = g_list_length (tasklist->priv->startup_sequences);
   n_grouped_buttons = 0;
   ungrouped_apps = g_list_copy (tasklist->priv->applications);
   score_set = FALSE;
@@ -767,7 +825,7 @@ wnck_tasklist_size_request  (GtkWidget      *widget,
   wnck_tasklist_layout (&fake_allocation,
 			tasklist->priv->max_button_width,
 			tasklist->priv->max_button_height,
-			n_windows,
+			n_windows + n_startup_sequences,
 			&n_cols, &n_rows);
 
   last_n_cols = G_MAXINT;
@@ -799,7 +857,7 @@ wnck_tasklist_size_request  (GtkWidget      *widget,
       wnck_tasklist_layout (&fake_allocation,
 			    tasklist->priv->max_button_width,
 			    tasklist->priv->max_button_height,
-			    n_windows - n_grouped_buttons,
+			    n_startup_sequences + n_windows - n_grouped_buttons,
 			    &n_cols, &n_rows);
       if (n_cols != last_n_cols &&
 	  (tasklist->priv->grouping == WNCK_TASKLIST_AUTO_GROUP ||
@@ -860,6 +918,7 @@ wnck_tasklist_size_allocate (GtkWidget      *widget,
   WnckTasklist *tasklist;
   WnckTask *app_task;
   int n_windows;
+  int n_startup_sequences;
   GList *l;
   int button_width;
   int total_width;
@@ -876,6 +935,7 @@ wnck_tasklist_size_allocate (GtkWidget      *widget,
   tasklist = WNCK_TASKLIST (widget);
 
   n_windows = g_list_length (tasklist->priv->windows);
+  n_startup_sequences = g_list_length (tasklist->priv->startup_sequences);
   n_grouped_buttons = 0;
   ungrouped_apps = g_list_copy (tasklist->priv->applications);
   score_set = FALSE;
@@ -887,7 +947,7 @@ wnck_tasklist_size_allocate (GtkWidget      *widget,
   button_width = wnck_tasklist_layout (allocation,
 				       tasklist->priv->max_button_width,
 				       tasklist->priv->max_button_height,
-				       n_windows,
+				       n_startup_sequences + n_windows,
 				       &n_cols, &n_rows);
   while (ungrouped_apps != NULL &&
 	 ((tasklist->priv->grouping == WNCK_TASKLIST_ALWAYS_GROUP) ||
@@ -927,7 +987,7 @@ wnck_tasklist_size_allocate (GtkWidget      *widget,
       button_width = wnck_tasklist_layout (allocation,
 					   tasklist->priv->max_button_width,
 					   tasklist->priv->max_button_height,
-					   n_windows - n_grouped_buttons,
+					   n_startup_sequences + n_windows - n_grouped_buttons,
 					   &n_cols, &n_rows);
     }
 
@@ -942,6 +1002,10 @@ wnck_tasklist_size_allocate (GtkWidget      *widget,
       l = l->next;
     }
 
+  /* Add all startup sequences */
+  visible_tasks = g_list_concat (visible_tasks, g_list_copy (tasklist->priv->startup_sequences));
+
+  /* Sort */
   visible_tasks = g_list_sort (visible_tasks, wnck_task_compare);
 
   /* Allocate children */
@@ -977,6 +1041,40 @@ wnck_tasklist_size_allocate (GtkWidget      *widget,
   GTK_WIDGET_CLASS (tasklist_parent_class)->size_allocate (widget, allocation);
 }
 
+static void
+wnck_tasklist_realize (GtkWidget *widget)
+{
+  WnckTasklist *tasklist;
+
+  tasklist = WNCK_TASKLIST (widget);  
+
+#ifdef HAVE_STARTUP_NOTIFICATION
+  tasklist->priv->sn_context =
+    sn_monitor_context_new (_wnck_screen_get_sn_display (tasklist->priv->screen),
+                            _wnck_screen_get_number (tasklist->priv->screen),
+                            wnck_tasklist_sn_event,
+                            tasklist,
+                            NULL);
+#endif
+  
+  (* GTK_WIDGET_CLASS (tasklist_parent_class)->realize) (widget);
+}
+
+static void
+wnck_tasklist_unrealize (GtkWidget *widget)
+{
+  WnckTasklist *tasklist;
+
+  tasklist = WNCK_TASKLIST (widget);
+
+#ifdef HAVE_STARTUP_NOTIFICATION
+  sn_monitor_context_unref (tasklist->priv->sn_context);
+  tasklist->priv->sn_context = NULL;
+#endif
+  
+  (* GTK_WIDGET_CLASS (tasklist_parent_class)->unrealize) (widget);
+}
+
 static void
 wnck_tasklist_forall (GtkContainer *container,
                       gboolean      include_internals,
@@ -1003,6 +1101,15 @@ wnck_tasklist_forall (GtkContainer *container,
       WnckTask *task = WNCK_TASK (tmp->data);
       tmp = tmp->next;
       
+      (* callback) (task->button, callback_data);
+    }
+
+  tmp = tasklist->priv->startup_sequences;
+  while (tmp != NULL)
+    {
+      WnckTask *task = WNCK_TASK (tmp->data);
+      tmp = tmp->next;
+      
       (* callback) (task->button, callback_data);
     }
 }
@@ -1058,6 +1165,26 @@ wnck_tasklist_remove (GtkContainer   *container,
 	  break;
 	}
     }
+
+  tmp = tasklist->priv->startup_sequences;
+  while (tmp != NULL)
+    {
+      WnckTask *task = WNCK_TASK (tmp->data);
+      tmp = tmp->next;
+      
+      if (task->button == widget)
+	{
+	  tasklist->priv->startup_sequences =
+	    g_list_remove (tasklist->priv->startup_sequences,
+			   task);
+
+          gtk_widget_unparent (widget);
+          g_object_unref (task);
+	  break;
+	}
+    }
+
+  gtk_widget_queue_resize (GTK_WIDGET (container));
 }
 
 static void
@@ -1261,6 +1388,8 @@ wnck_tasklist_update_lists (WnckTasklist *tasklist)
   
   /* since we cleared active_window we need to reset it */
   wnck_tasklist_active_window_changed (tasklist->priv->screen, tasklist);
+
+  gtk_widget_queue_resize (GTK_WIDGET (tasklist));
 }
 
 static void
@@ -1270,6 +1399,9 @@ wnck_tasklist_change_active_task (WnckTasklist *tasklist, WnckTask *active_task)
       active_task == tasklist->priv->active_task)
     return;
 
+  g_assert (active_task == NULL ||
+            active_task->type != WNCK_TASK_STARTUP_SEQUENCE);
+  
   if (tasklist->priv->active_task)
     {
       tasklist->priv->active_task->really_toggling = TRUE;
@@ -1387,7 +1519,8 @@ wnck_tasklist_window_changed_workspace (WnckWindow   *window,
     {
       WnckTask *task = l->data;
 
-      if (!task->is_application && task->window == window)
+      if (task->type == WNCK_TASK_WINDOW &&
+          task->window == window)
         need_update = TRUE;
 
       l = l->next;
@@ -1504,7 +1637,7 @@ wnck_tasklist_activate_task_window     (WnckTask *task)
 	  !tasklist->priv->switch_workspace_on_unminimize)
 	wnck_workspace_activate (window_ws);
 	  
-      wnck_window_activate (task->window);
+      wnck_window_activate_transient (task->window);
     }
   else
     {
@@ -1522,7 +1655,7 @@ wnck_tasklist_activate_task_window     (WnckTask *task)
           if (window_ws)
             wnck_workspace_activate (window_ws);
 
-	  wnck_window_activate (task->window);
+	  wnck_window_activate_transient (task->window);
 	}
     }
   
@@ -1629,20 +1762,23 @@ wnck_task_button_toggled (GtkButton *button,
 				!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)));
   task->really_toggling = FALSE;
 
-  if (task->is_application)
+  switch (task->type)
     {
+    case WNCK_TASK_APPLICATION:
       wnck_task_popup_menu (task, FALSE);
-    }
-  else
-    {
+      break;
+    case WNCK_TASK_WINDOW:
       if (task->window == NULL)
 	return;
-
+      
       wnck_tasklist_activate_task_window (task);
 
+      /* FIXME why is this here? WTF? */
       state = wnck_window_get_state (task->window);
+      break;
+    case WNCK_TASK_STARTUP_SEQUENCE:
+      break;
     }
-  
 }
 
 static char *
@@ -1651,14 +1787,15 @@ wnck_task_get_text (WnckTask *task)
   WnckWindowState state;
   const char *name;
   
-  if (task->is_application)
+  switch (task->type)
     {
+    case WNCK_TASK_APPLICATION:
       return g_strdup_printf ("%s (%d)",
                               wnck_application_get_icon_name (task->application),
                               g_list_length (task->windows));
-    }
-  else
-    {
+      break;
+
+    case WNCK_TASK_WINDOW:
       state = wnck_window_get_state (task->window);
       name = wnck_window_get_icon_name (task->window);
       
@@ -1666,7 +1803,23 @@ wnck_task_get_text (WnckTask *task)
 	return g_strdup_printf ("[%s]", name);
       else
 	return g_strdup (name);
+      break;
+
+    case WNCK_TASK_STARTUP_SEQUENCE:
+#ifdef HAVE_STARTUP_NOTIFICATION
+      /* this may return NULL, but probably won't */
+      name = sn_startup_sequence_get_name (task->startup_sequence);
+      if (name == NULL)
+        name = sn_startup_sequence_get_binary_name (task->startup_sequence);
+      
+      return g_strdup (name);
+#else
+      return NULL;
+#endif
+      break;
     }
+
+  return NULL;
 }
 
 static void
@@ -1755,18 +1908,27 @@ wnck_task_get_icon (WnckTask *task)
 {
   WnckWindowState state;
   GdkPixbuf *pixbuf;
+
+  pixbuf = NULL;
   
-  if (task->is_application)
+  switch (task->type)    
     {
+    case WNCK_TASK_APPLICATION:
       pixbuf =  wnck_task_scale_icon (wnck_application_get_mini_icon (task->application),
 				      FALSE);
-    }
-  else
-    {
+      break;
+    case WNCK_TASK_WINDOW:
       state = wnck_window_get_state (task->window);
 
       pixbuf =  wnck_task_scale_icon (wnck_window_get_mini_icon (task->window),
 				      state & WNCK_WINDOW_STATE_MINIMIZED);
+      break;
+    case WNCK_TASK_STARTUP_SEQUENCE:
+      /* FIXME we have to load an icon from the icon loader,
+       * and before that we need to just use a generic hourglass
+       * or something
+       */
+      break;
     }
 
   return pixbuf;
@@ -1786,9 +1948,12 @@ wnck_task_update_visible_state (WnckTask *task)
     g_object_unref (pixbuf);
 
   text = wnck_task_get_text (task);
-  gtk_label_set_text (GTK_LABEL (task->label), text);
-  gtk_tooltips_set_tip (task->tasklist->priv->tooltips, task->button, text, NULL);
-  g_free (text);
+  if (text != NULL)
+    {
+      gtk_label_set_text (GTK_LABEL (task->label), text);
+      gtk_tooltips_set_tip (task->tasklist->priv->tooltips, task->button, text, NULL);
+      g_free (text);
+    }
 
   gtk_widget_queue_resize (GTK_WIDGET (task->tasklist));
 }
@@ -1865,7 +2030,7 @@ wnck_task_motion_timeout (gpointer data)
 
   task->button_activate = 0;
 
-  wnck_window_activate (task->window);
+  wnck_window_activate_transient (task->window);
 
   return FALSE;
 }
@@ -1892,7 +2057,7 @@ wnck_task_drag_motion (GtkWidget          *widget,
 		       WnckTask            *task)
 {
 
-   if (task->button_activate == 0 && !(task->is_application))
+   if (task->button_activate == 0 && task->type == WNCK_TASK_WINDOW)
        task->button_activate = g_timeout_add (TIMEOUT_ACTIVATE,
                                               wnck_task_motion_timeout,
                                               task);
@@ -1908,46 +2073,53 @@ wnck_task_button_press_event (GtkWidget	      *widget,
 {
   WnckTask *task = WNCK_TASK (data);
 
-  if (task->is_application)
+  switch (task->type)
     {
+    case WNCK_TASK_APPLICATION:
       wnck_task_popup_menu (task,
                             event->button == 3);
       return TRUE;
-    }
-  else if (event->button == 1)
-    {
-      if (wnck_window_is_active (task->window))
-        task->was_active = TRUE;
-      else
-        task->was_active = FALSE;
+      break;
 
-      return FALSE;
-    } 
-  else if (event->button == 3)
-    {
-      if (task->action_menu)
-        gtk_widget_destroy (task->action_menu);
+    case WNCK_TASK_WINDOW:
+      if (event->button == 1)
+        {
+          if (wnck_window_is_active (task->window))
+            task->was_active = TRUE;
+          else
+            task->was_active = FALSE;
 
-      g_assert (task->action_menu == NULL);
+          return FALSE;
+        } 
+      else if (event->button == 3)
+        {
+          if (task->action_menu)
+            gtk_widget_destroy (task->action_menu);
+
+          g_assert (task->action_menu == NULL);
       
-      task->action_menu = wnck_create_window_action_menu (task->window);
+          task->action_menu = wnck_create_window_action_menu (task->window);
 
-      g_object_add_weak_pointer (G_OBJECT (task->action_menu),
-                                 (void**) &task->action_menu);
+          g_object_add_weak_pointer (G_OBJECT (task->action_menu),
+                                     (void**) &task->action_menu);
       
-      gtk_menu_set_screen (GTK_MENU (task->action_menu),
-                           _wnck_screen_get_gdk_screen (task->tasklist->priv->screen));
+          gtk_menu_set_screen (GTK_MENU (task->action_menu),
+                               _wnck_screen_get_gdk_screen (task->tasklist->priv->screen));
 
-      gtk_widget_show (task->action_menu);
-      gtk_menu_popup (GTK_MENU (task->action_menu),
-                      NULL, NULL,
-                      wnck_task_position_menu, task->button,
-                      event->button,
-                      gtk_get_current_event_time ());
+          gtk_widget_show (task->action_menu);
+          gtk_menu_popup (GTK_MENU (task->action_menu),
+                          NULL, NULL,
+                          wnck_task_position_menu, task->button,
+                          event->button,
+                          gtk_get_current_event_time ());
 
-      /* FIXME we should really arrange to destroy the menu on popdown */
+          /* FIXME we should really arrange to destroy the menu on popdown */
       
-      return TRUE;
+          return TRUE;
+        }
+      break;
+    case WNCK_TASK_STARTUP_SEQUENCE:
+      break;
     }
 
   return FALSE;
@@ -1965,7 +2137,10 @@ wnck_task_create_widgets (WnckTask *task)
   if (!disable_sound_quark)
     disable_sound_quark = g_quark_from_static_string ("gnome_disable_sound_events");
 
-  task->button = gtk_toggle_button_new ();
+  if (task->type == WNCK_TASK_STARTUP_SEQUENCE)
+    task->button = gtk_button_new ();
+  else
+    task->button = gtk_toggle_button_new ();
   task->button_activate = 0;
   g_object_set_qdata (G_OBJECT (task->button),
 		      disable_sound_quark, GINT_TO_POINTER (TRUE));
@@ -2014,10 +2189,11 @@ wnck_task_create_widgets (WnckTask *task)
   g_free (text);
   
   /* Set up signals */
-  g_signal_connect_object (G_OBJECT (task->button), "toggled",
-                           G_CALLBACK (wnck_task_button_toggled),
-                           G_OBJECT (task),
-                           0);
+  if (GTK_IS_TOGGLE_BUTTON (task->button))
+    g_signal_connect_object (G_OBJECT (task->button), "toggled",
+                             G_CALLBACK (wnck_task_button_toggled),
+                             G_OBJECT (task),
+                             0);
 
   
   g_signal_connect_object (G_OBJECT (task->button), "button_press_event",
@@ -2035,21 +2211,25 @@ wnck_task_create_widgets (WnckTask *task)
                            G_OBJECT (task),
                            0);
 
-  if (task->is_application)
+  switch (task->type)
     {
+    case WNCK_TASK_APPLICATION:
       task->app_name_changed_tag = g_signal_connect (G_OBJECT (task->application), "name_changed",
                                                      G_CALLBACK (wnck_task_app_name_changed), task);
-    }
-  else
-    {
+      break;
+      
+    case WNCK_TASK_WINDOW:
       task->state_changed_tag = g_signal_connect (G_OBJECT (task->window), "state_changed",
                                                   G_CALLBACK (wnck_task_state_changed), task->tasklist);
       task->icon_changed_tag = g_signal_connect (G_OBJECT (task->window), "icon_changed",
                                                  G_CALLBACK (wnck_task_icon_changed), task);
       task->name_changed_tag = g_signal_connect (G_OBJECT (task->window), "name_changed",
 						 G_CALLBACK (wnck_task_name_changed), task);
-    }
+      break;
 
+    case WNCK_TASK_STARTUP_SEQUENCE:
+      break;
+    }
 }
 
 static void
@@ -2094,23 +2274,37 @@ wnck_task_compare (gconstpointer  a,
   WnckTask *task2 = WNCK_TASK (b);
   gulong xid1_1, xid1_2;
   gulong xid2_1, xid2_2;
-
-  if (task1->is_application)
-    xid1_1 = xid1_2 = wnck_application_get_xid (task1->application);
-  else
+  
+  xid1_1 = xid1_2 = xid2_1 = xid2_2 = 0;   /* silence compiler */
+  
+  switch (task1->type)
     {
+    case WNCK_TASK_APPLICATION:
+      xid1_1 = xid1_2 = wnck_application_get_xid (task1->application);
+      break;
+    case WNCK_TASK_WINDOW:
       xid1_1 = wnck_window_get_group_leader (task1->window);
       xid1_2 = wnck_window_get_xid (task1->window);
+      break;
+    case WNCK_TASK_STARTUP_SEQUENCE:
+      xid1_1 = xid1_2 = G_MAXULONG;
+      break;
     }
 
-  if (task2->is_application)
-    xid2_1 = xid2_2 = wnck_application_get_xid (task2->application);
-  else
+  switch (task2->type)
     {
+    case WNCK_TASK_APPLICATION:
+      xid2_1 = xid2_2 = wnck_application_get_xid (task2->application);
+      break;
+    case WNCK_TASK_WINDOW:
       xid2_1 = wnck_window_get_group_leader (task2->window);
       xid2_2 = wnck_window_get_xid (task2->window);
+      break;
+    case WNCK_TASK_STARTUP_SEQUENCE:
+      xid2_1 = xid2_2 = G_MAXULONG;
+      break;
     }
-    
+      
   if ((xid1_1 < xid2_1) || ((xid1_1 == xid2_1) && (xid1_2 < xid2_2)))
     return -1;
   else if ((xid1_1 == xid2_1) && (xid1_2 == xid2_2))
@@ -2127,7 +2321,7 @@ wnck_task_new_from_window (WnckTasklist *tasklist,
 
   task = g_object_new (WNCK_TYPE_TASK, NULL);
 
-  task->is_application = FALSE;
+  task->type = WNCK_TASK_WINDOW;
   task->window = g_object_ref (window);
   task->application = g_object_ref (wnck_window_get_application (window));
   task->tasklist = tasklist;
@@ -2145,7 +2339,7 @@ wnck_task_new_from_application (WnckTasklist    *tasklist,
 
   task = g_object_new (WNCK_TYPE_TASK, NULL);
 
-  task->is_application = TRUE;
+  task->type = WNCK_TASK_APPLICATION;
   task->application = g_object_ref (application);
   task->window = NULL;
   task->tasklist = tasklist;
@@ -2159,3 +2353,83 @@ wnck_task_new_from_application (WnckTasklist    *tasklist,
   
   return task;
 }
+
+#ifdef HAVE_STARTUP_NOTIFICATION
+static WnckTask*
+wnck_task_new_from_startup_sequence (WnckTasklist      *tasklist,
+                                     SnStartupSequence *sequence)
+{
+  WnckTask *task;
+
+  task = g_object_new (WNCK_TYPE_TASK, NULL);
+
+  task->type = WNCK_TASK_STARTUP_SEQUENCE;
+  task->application = NULL;
+  task->window = NULL;
+  task->startup_sequence = sequence;
+  sn_startup_sequence_ref (task->startup_sequence);
+  task->tasklist = tasklist;
+  
+  wnck_task_create_widgets (task);
+
+  return task;
+}
+
+static void
+wnck_tasklist_sn_event (SnMonitorEvent *event,
+                        void           *user_data)
+{
+  WnckTasklist *tasklist;
+
+  tasklist = WNCK_TASKLIST (user_data);
+
+  switch (sn_monitor_event_get_type (event))
+    {
+    case SN_MONITOR_EVENT_INITIATED:
+      {
+        WnckTask *task;
+        
+        task = wnck_task_new_from_startup_sequence (tasklist,
+                                                    sn_monitor_event_get_startup_sequence (event));
+        
+        gtk_widget_set_parent (task->button, GTK_WIDGET (tasklist));
+        gtk_widget_show (task->button);
+	      
+        tasklist->priv->startup_sequences =
+          g_list_prepend (tasklist->priv->startup_sequences,
+                          task);
+
+        gtk_widget_queue_resize (GTK_WIDGET (tasklist));
+      }
+      break;
+
+    case SN_MONITOR_EVENT_COMPLETED:
+      {
+        GList *tmp;
+        tmp = tasklist->priv->startup_sequences;
+        while (tmp != NULL)
+          {
+            WnckTask *task = WNCK_TASK (tmp->data);
+
+            if (task->startup_sequence ==
+                sn_monitor_event_get_startup_sequence (event))
+              {
+                g_assert (task->button != NULL);
+                /* removes task from list as a side effect */
+                gtk_widget_destroy (task->button);      
+                break;
+              }
+            
+            tmp = tmp->next;
+          }
+      }
+      break;
+
+    case SN_MONITOR_EVENT_CHANGED:
+      break;
+
+    case SN_MONITOR_EVENT_CANCELED:
+      break;
+    }
+}
+#endif /* HAVE_STARTUP_NOTIFICATION */
