@@ -29,7 +29,8 @@ typedef enum
   MAXIMIZE,
   SHADE,
   MOVE,
-  RESIZE
+  RESIZE,
+  PIN
 } WindowAction;
 
 typedef struct _ActionMenuData ActionMenuData;
@@ -38,12 +39,13 @@ struct _ActionMenuData
 {
   WnckWindow *window;
   GtkWidget *menu;
-  GtkWidget *close_item;
   GtkWidget *minimize_item;
   GtkWidget *maximize_item;
   GtkWidget *shade_item;
   GtkWidget *move_item;
   GtkWidget *resize_item;
+  GtkWidget *close_item;
+  GtkWidget *pin_item;
   guint idle_handler;
 };
 
@@ -129,6 +131,12 @@ item_activated_callback (GtkWidget *menu_item,
     case RESIZE:
       wnck_window_keyboard_size (amd->window);
       break;
+    case PIN:
+      if (wnck_window_is_pinned (amd->window))
+        wnck_window_unpin (amd->window);
+      else
+        wnck_window_pin (amd->window);
+      break;
     }
 }
 
@@ -141,6 +149,34 @@ set_item_text (GtkWidget  *mi,
   gtk_label_set_use_underline (GTK_LABEL (GTK_BIN (mi)->child), TRUE);
 }
 
+static void
+set_item_stock (GtkWidget  *mi,
+                const char *stock_id)
+{
+  GtkWidget *image;
+  
+  image = gtk_image_menu_item_get_image (GTK_IMAGE_MENU_ITEM (mi));
+
+  if (stock_id == NULL)
+    {
+      if (image != NULL)
+        gtk_widget_destroy (image);
+      return;
+    }
+
+  if (image == NULL)
+    {
+      image = gtk_image_new_from_stock (stock_id, GTK_ICON_SIZE_MENU);
+      gtk_widget_show (image);
+      gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mi), image);
+    }
+  else
+    {
+      gtk_image_set_from_stock (GTK_IMAGE (image), stock_id,
+                                GTK_ICON_SIZE_MENU);
+    }
+}
+
 static gboolean
 update_menu_state (ActionMenuData *amd)
 {
@@ -153,12 +189,14 @@ update_menu_state (ActionMenuData *amd)
   if (wnck_window_is_minimized (amd->window))
     {
       set_item_text (amd->minimize_item, _("Un_minimize"));
+      set_item_stock (amd->minimize_item, NULL);
       gtk_widget_set_sensitive (amd->minimize_item,
                                 (actions & WNCK_WINDOW_ACTION_UNMINIMIZE) != 0);
     }
   else
     {
       set_item_text (amd->minimize_item, _("_Minimize"));
+      set_item_stock (amd->minimize_item, WNCK_STOCK_MINIMIZE);
       gtk_widget_set_sensitive (amd->minimize_item,
                                 (actions & WNCK_WINDOW_ACTION_MINIMIZE) != 0);
     }
@@ -166,29 +204,48 @@ update_menu_state (ActionMenuData *amd)
   if (wnck_window_is_maximized (amd->window))
     {
       set_item_text (amd->maximize_item, _("_Unmaximize"));
+      set_item_stock (amd->maximize_item, NULL);
       gtk_widget_set_sensitive (amd->maximize_item,
                                 (actions & WNCK_WINDOW_ACTION_UNMAXIMIZE) != 0);
     }
   else
     {
       set_item_text (amd->maximize_item, _("Ma_ximize"));
+      set_item_stock (amd->maximize_item, WNCK_STOCK_MAXIMIZE);
       gtk_widget_set_sensitive (amd->maximize_item,
                                 (actions & WNCK_WINDOW_ACTION_MAXIMIZE) != 0);
     }
 
   if (wnck_window_is_shaded (amd->window))
     {
-      set_item_text (amd->shade_item, _("U_nshade"));
+      set_item_text (amd->shade_item, _("_Unroll"));
+      set_item_stock (amd->shade_item, NULL);
       gtk_widget_set_sensitive (amd->shade_item,
                                 (actions & WNCK_WINDOW_ACTION_UNSHADE) != 0);
     }
   else
     {
-      set_item_text (amd->shade_item, _("_Shade"));
+      set_item_text (amd->shade_item, _("Roll _Up"));
+      set_item_stock (amd->shade_item, NULL);
       gtk_widget_set_sensitive (amd->shade_item,
                                 (actions & WNCK_WINDOW_ACTION_SHADE) != 0);
     }
 
+  if (wnck_window_is_pinned (amd->window))
+    {
+      set_item_text (amd->pin_item, _("Only on _This Workspace"));
+      set_item_stock (amd->pin_item, NULL);
+      gtk_widget_set_sensitive (amd->pin_item,
+                                (actions & WNCK_WINDOW_ACTION_CHANGE_WORKSPACE) != 0);
+    }
+  else
+    {
+      set_item_text (amd->pin_item, _("Put on _All Workspaces"));
+      set_item_stock (amd->pin_item, NULL);
+      gtk_widget_set_sensitive (amd->pin_item,
+                                (actions & WNCK_WINDOW_ACTION_CHANGE_WORKSPACE) != 0);
+    }
+  
   gtk_widget_set_sensitive (amd->close_item,
                             (actions & WNCK_WINDOW_ACTION_CLOSE) != 0);
   
@@ -242,8 +299,8 @@ make_menu_item (ActionMenuData *amd,
 {
   GtkWidget *mi;
   
-  mi = gtk_menu_item_new_with_label ("");
-
+  mi = gtk_image_menu_item_new_with_label ("");
+  
   set_data (G_OBJECT (mi), amd);
   
   g_signal_connect (G_OBJECT (mi), "activate",
@@ -277,7 +334,10 @@ wnck_create_window_action_menu (WnckWindow *window)
 {
   GtkWidget *menu;
   ActionMenuData *amd;
-
+  GtkWidget *separator;
+  
+  _wnck_stock_icons_init ();
+  
   amd = g_new0 (ActionMenuData, 1);
   amd->window = window;
   
@@ -290,13 +350,6 @@ wnck_create_window_action_menu (WnckWindow *window)
   g_object_weak_ref (G_OBJECT (window), window_weak_notify, menu);
   g_object_weak_ref (G_OBJECT (menu), object_weak_notify, window);
   
-  amd->close_item = make_menu_item (amd, CLOSE);
-  
-  gtk_menu_shell_append (GTK_MENU_SHELL (menu),
-                         amd->close_item);
-
-  set_item_text (amd->close_item, _("_Close"));
-  
   amd->minimize_item = make_menu_item (amd, MINIMIZE);
   
   gtk_menu_shell_append (GTK_MENU_SHELL (menu),
@@ -317,12 +370,37 @@ wnck_create_window_action_menu (WnckWindow *window)
                          amd->move_item);  
 
   set_item_text (amd->move_item, _("Mo_ve"));
+  set_item_stock (amd->move_item, NULL);
   
   amd->resize_item = make_menu_item (amd, RESIZE);
   gtk_menu_shell_append (GTK_MENU_SHELL (menu),
                          amd->resize_item);
 
   set_item_text (amd->resize_item, _("_Resize"));
+  set_item_stock (amd->move_item, NULL);
+
+  separator = gtk_separator_menu_item_new ();
+  gtk_widget_show (separator);
+  gtk_menu_shell_append (GTK_MENU_SHELL (menu),
+                         separator);
+
+  amd->close_item = make_menu_item (amd, CLOSE);
+  
+  gtk_menu_shell_append (GTK_MENU_SHELL (menu),
+                         amd->close_item);
+
+  set_item_text (amd->close_item, _("_Close"));
+  set_item_stock (amd->close_item, WNCK_STOCK_DELETE);
+
+  separator = gtk_separator_menu_item_new ();
+  gtk_widget_show (separator);
+  gtk_menu_shell_append (GTK_MENU_SHELL (menu),
+                         separator);
+
+  amd->pin_item = make_menu_item (amd, PIN);
+  gtk_menu_shell_append (GTK_MENU_SHELL (menu),
+                         amd->pin_item);
+  set_item_stock (amd->pin_item, NULL);
   
   g_signal_connect_object (G_OBJECT (amd->window), 
                            "state_changed",
