@@ -595,6 +595,40 @@ _wnck_get_utf8_list (Window   xwindow,
   return retval;
 }
 
+void
+_wnck_set_utf8_list (Window   xwindow,
+                     Atom     atom,
+                     char   **list)
+{
+  Atom utf8_string;
+  GString *flattened;
+  int i;
+  
+  utf8_string = _wnck_atom_get ("UTF8_STRING");  
+
+  /* flatten to nul-separated list */
+  flattened = g_string_new ("");
+  i = 0;
+  while (list[i] != NULL)
+    {
+      g_string_append_len (flattened, list[i],
+                           strlen (list[i]) + 1);
+      ++i;
+    }
+
+  _wnck_error_trap_push ();
+  
+  XChangeProperty (gdk_display,
+		   xwindow,
+                   atom,
+		   utf8_string, 8, PropModeReplace,
+		   flattened->str, flattened->len);
+  
+  _wnck_error_trap_pop ();
+
+  g_string_free (flattened, TRUE);
+}
+
 void
 _wnck_error_trap_push (void)
 {
@@ -662,6 +696,7 @@ filter_func (GdkXEvent  *gdkxevent,
              gpointer    data)
 {
   XEvent *xevent = gdkxevent;
+  int i;
   
   switch (xevent->type)
     {
@@ -705,6 +740,26 @@ filter_func (GdkXEvent  *gdkxevent,
         _wnck_desktop_layout_manager_process_event (xevent);
       }
       break;
+
+    case ClientMessage:
+#ifdef HAVE_STARTUP_NOTIFICATION
+      /* We're cheating as officially libsn requires
+       * us to send all events through sn_display_process_event
+       */
+      i = 0;
+      while (i < ScreenCount (gdk_display))
+        {
+          WnckScreen *s;
+
+          s = _wnck_screen_get_existing (i);
+          if (s != NULL)
+            sn_display_process_event (_wnck_screen_get_sn_display (s),
+                                      xevent);
+          
+          ++i;
+        }
+#endif /* HAVE_STARTUP_NOTIFICATION */
+      break;
     }
   
   return GDK_FILTER_CONTINUE;
